<?php

/**
 * iToggle ajax functions
 */

/**
 * Page callback
 * 
 * @see itoggle_menu().
 */
function itoggle_ajax_callback() {
  // extract values from POST
  $token = filter_input(INPUT_POST, 'token');
  $type = filter_input(INPUT_POST, 'type');
  $property = filter_input(INPUT_POST, 'property');
  $id = filter_input(INPUT_POST, 'id');
  $value = filter_input(INPUT_POST, 'value');
  $scope = filter_input(INPUT_POST, 'scope');
  // generate token key
  $token_key = "itoggle_{$type}_{$property}_{$id}";

  // router
  if (!empty($token) && drupal_valid_token($token, $token_key)) {
    switch ($scope) {
      case 'entity':
        $ok = _itoggle_ajax_toggle_entity($type, $property, $id, $value);
        break;
      case 'field':
        $ok = _itoggle_ajax_toggle_field($type, $property, $id, $value);
        break;
      // should never really get here
      case 'field-edit':
        $ok = TRUE;
        break;
    }
  }
  else {
    // let $ok be NULL so we know the difference between FALSE (error) and UNDEFINDED (invalid token)
    $ok = NULL;
  }

  drupal_json_output(array('ok' => $ok));
  exit;
}

/**
 * Helper function
 * Changes field values
 * 
 * @param string $type
 * @param string $property
 * @param int $id
 * @param int $value
 * @param int $delta
 */
function _itoggle_ajax_toggle_field($type, $property, $id, $value, $delta = 0) {
  // check permissions
  $access = FALSE;

  // @TODO check permissions
  $access = TRUE;

  if ($access) {
    $entity = current(entity_load($type, array($id)));

    if ($entity) {
      $lang = $entity->language;
      $entity->{$property}[$lang][$delta] = $value;
      $return = entity_save($type, $entity);
      return $return !== FALSE;
    }
  }

  return FALSE;
}

/**
 * Helper function
 * Changes entity properties
 * 
 * @param string $type
 * @param string $property
 * @param int $id
 * @param int $value
 */
function _itoggle_ajax_toggle_entity($type, $property, $id, $value) {
  // check permissions
  $access = FALSE;

  // @TODO check permissions
  $acecss = TRUE;
  if ($access) {
    // @TODO if node, check if override_node_options exists?
    if (module_exists('override_node_options')) {
      $node = node_load($id);
      $type = $node->type;
      $access = user_access("override $type published option");
    }
  }
  else {
    $access = TRUE;
  }

  if ($access === TRUE) {
    $entity = current(entity_load($type, array($id)));

    if ($entity) {
      $entity->{$property} = $value;
      $return = entity_save($type, $entity);
      return ($return !== FALSE);
    }
  }

  return FALSE;
}