<?php

/**
 * @file
 * iToggle theme functions
 */

/**
 * Implements hook_theme().
 */
function itoggle_theme($existing, $type, $theme, $path) {
  return array(
    'itoggle' => array(
      'variables' => array(
        'type' => NULL,
        'id' => NULL,
        'property' => NULL,
        'checked' => NULL,
        'scope' => NULL,
        'clickable' => 1,
        'display_type' => 0,
      ),
    ),
  );
}

/**
 * Theme callback
 * 
 * @param array $variables
 *  An array with the folowing keys/values:
 *    - string $type
 *        The entity type
 *    - int $id
 *        The property id
 *    - string $property
 *        The property machine-name
 *    - bool $checked
 *        The property value (true/false)
 *    - string $scope
 *        The widget scope (views/field/inline)
 *    - bool $clickable
 *        Whether the widget is clickable (triggers callback)
 *    - int $display_type
 *        The iToggle display type (0 = on/off, 1 = yes/no)
 * @see itoggle_theme().
 */
function theme_itoggle($variables) {
  extract($variables);

  $checked = ($checked == TRUE) ? 'checked="checked"' : '';
  $element_id = "itoggle_{$type}_{$property}_{$id}";
  $display_type = ($display_type == 0) ? 'onoff' : 'yesno';

  return <<<HTML
<div class="itoggle-wrapper itoggle-display-{$display_type} itoggle-{$type} itoggle-{$type}_{$property}" data-id="{$id}" data-type="{$type}" data-scope="{$scope}" data-property="{$property}">
  <input type="checkbox" id="{$element_id}" {$checked} />
</div>
HTML;
}

/**
 * Implements hook_preprocess_hook().
 * 
 * Includes javascript files and settings for theme_itoggle().
 * 
 * @see itoggle_theme().
 */
function itoggle_preprocess_itoggle(&$variables) {
  extract($variables);

  // set token key
  $token_key = "itoggle_{$type}_{$property}_{$id}";

  // include itoggle javascript & styles
  itoggle_include_itoggle(TRUE);

  // add an individual token for each combination
  drupal_add_js(array(
    'itoggle' => array(
      'tokens' => array(
        $token_key => drupal_get_token($token_key),
      ),
      'clickable' => array(
        $token_key => (bool) $clickable,
      ),
    )
  ), 'setting');
}