<?php

/**
 * @file
 * View handler for iToggle Views
 */

/**
 * A handler to provide a custom field
 *
 * @ingroup views_field_handlers
 */
class itoggle_views_handler_field extends views_handler_field {

  protected $_itoggle;

  // I have found no place to add css/js includes that executes just once per view
  // all functions here will be executed once per itoggle field
  // we use a static cache to avoid this
  function init(&$view, &$options) {
    // find out the field name (ie what entity type and what property)
    $field_name = str_replace('itoggle_', '', $options['id']);
    list ($type, $property) = explode('_', $field_name);

    // store info in instance variable
    $this->_itoggle = array(
      'info' => itoggle_get_entity_info(),
      'type' => $type,
      'property' => $property,
    );

    parent::init($view, $options);
  }

  function query() {
    // Do nothing, as this handler does not need to do anything to the query itself.
  }

  /**
   * Render the trigger field and its linked popup information.
   */
  function render($values) {
    $type = $this->_itoggle['type'];
    $property = $this->_itoggle['property'];
    $id = $this->_itoggle['info'][$type]['entity keys']['id'];

    if (isset($values->{$id})) {
      $id = $values->{$id};

      // check if property value already exists in $values array before loading entity
      if (array_key_exists($property, $values)) {
        $value = $values[$property];
      }
      else if (array_key_exists("{$type}_{$property}", $values)) {
        $value = $values["{$type}_{$property}"];
      }
      // no go, gotta load the entity
      else {
        $entity = current(entity_load($type, array($id)));
        $value = $entity->{$property};
      }

      $clickable = $this->options['itoggle_clickable'];
      $display_type = $this->options['itoggle_display_type'];

      return theme('itoggle', array(
          'type' => $type,
          'id' => $id,
          'property' => $property,
          'checked' => (bool) $value,
          'scope' => 'entity',
          'clickable' => $clickable,
          'display_type' => $display_type,
        ));
    }
    return NULL;
  }

  /**
   * Called to determine what to tell the clicksorter.
   */
  function click_sort($order) {
    $property = $this->_itoggle['property'];

    if (isset($this->field_alias)) {
      // Since fields should always have themselves already added, just
      // add a sort on the field.
      $params = $this->options['group_type'] != 'group' ? array('function' => $this->options['group_type']) : array();
      $this->query->add_orderby(NULL, NULL, $order, $property, $params);
    }
  }

  /**
   * Define extra iToggle options
   */
  function option_definition() {
    $options = parent::option_definition();
    $options['itoggle_clickable'] = array('default' => TRUE, 'bool' => TRUE);
    $options['itoggle_display_type'] = array('default' => 0);
    return $options;
  }

  /**
   * Extra iToggle options form
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $form['itoggle'] = array(
      '#type' => 'fieldset',
      '#title' => t('iToggle'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#weight' => -1,
      '#tree' => TRUE,
    );

    // prefix options with module name to avoid conflicts
    $clickable = $this->options['itoggle_clickable'];
    $display_type = $this->options['itoggle_display_type'];
    $options_form = itoggle_get_options_form($clickable, $display_type);
    
    // add extra messages in Views UI
    $options_form['warning_views'] = array(
      '#markup' => '<p><strong>' . t('You will have to save the View and refresh the page to see the iToggle widget.') . '</strong></p>',
      '#weight' => -4,
    );
    $options_form['clickable']['#description'] .= '<br/><strong>' . t("Regardless of your choice, the widget won't be clickable in the Views preview.") . '</strong>';
    
    // wrap the form in a fieldset
    foreach ($options_form as $key => $value) {
      $form['itoggle'][$key] = $value;
    }
  }

  /**
   * Extra iToggle options submit callback
   */
  function options_submit(&$form, &$form_state) {
    parent::options_submit($form, $form_state);
    $options = $form_state['values']['options']['itoggle'];
    unset($form_state['values']['options']['itoggle']);
    // prefix options with module name to avoid conflicts
    $form_state['values']['options']['itoggle_clickable'] = $options['clickable'];
    $form_state['values']['options']['itoggle_display_type'] = $options['display_type'];
  }

}